@@ -23,7 +23,7 @@ class Zone:
         #Set on create or exists call
         self.id = None
         self.xml_header = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + \
-            "<ChangeResourceRecordSetsRequest xmlns=\"https://route53.amazonaws.com/doc/2010-10-01/\">\n" + \
+            "<ChangeResourceRecordSetsRequest xmlns=\"https://route53.amazonaws.com/doc/2011-05-05/\">\n" + \
             " <ChangeBatch>\n" + "  <Comment>Updates to Zone " + self.zone_name + "</Comment>\n" + "  <Changes>\n"
         self.xml_footer = "  </Changes>\n" + " </ChangeBatch>\n" + "</ChangeResourceRecordSetsRequest>\n"
 
@@ -36,10 +36,32 @@ class Zone:
         tmp = tempfile.TemporaryFile()
         dnszone.to_file(tmp)
         tmp.seek(0)
-        return "Zone " + self.zone_name + "\n" + tmp.read()
+        return "Zone " + self.zone_name + " ID: " + self.id + "\n" + tmp.read()
+
+    def _change_alias_xml(self, action, name, rvalue):
+        """Returns amazon change xml for a specific route 53 entry."""
+        rtype = 'A'
+        words = rvalue.split()
+        xmlout = "   <Change>\n" + "    <Action>" + action + "</Action>\n" + "     <ResourceRecordSet>\n" + \
+            "      <Name>" + name + "</Name>\n" + "      <Type>" + rtype + "</Type>\n" + \
+            "      <AliasTarget>\n            <HostedZoneId>" + words[1] + "</HostedZoneId>\n" + \
+            "            <DNSName>" + words[2][:-1] + "</DNSName>\n      </AliasTarget>\n" + \
+            "     </ResourceRecordSet>\n" + "   </Change>\n"
+        return xmlout
 
     def _change_xml(self, action, name, rtype, ttl, values):
         """Returns amazon Change xml for an action on an entry."""
+        #Check to see if any TXT entries are a masked route53 alias
+        if rtype == 'TXT':
+            for rvalue in values:
+                if rvalue[1:7] == 'Alias ':
+                    log.debug('Interpreting TXT entry as a route53 alias.')
+                    xmlout = self._change_alias_xml(action, name, rvalue)
+                    if len(values) > 1:
+                        values.remove(rvalue)
+                        xmlout += self._change_xml(action, name, rtype, ttl, values)
+                    return xmlout
+        
         xmlout = "   <Change>\n" + "    <Action>" + action + "</Action>\n" + "     <ResourceRecordSet>\n" + \
             "      <Name>" + name + "</Name>\n" + "      <Type>" + rtype + "</Type>\n" + \
             "      <TTL>" + str(ttl) + "</TTL>\n" + "      <ResourceRecords>\n"
@@ -47,7 +69,7 @@ class Zone:
         for rvalue in values:
             xmlout += "       <ResourceRecord>\n" + "        <Value>" + rvalue + "</Value>\n" + \
             "       </ResourceRecord>\n" 
-            
+        
         xmlout += "     </ResourceRecords>\n" + "    </ResourceRecordSet>\n" + "   </Change>\n"
         return xmlout
 
@@ -125,17 +147,17 @@ class Zone:
                 from_records[key].sort() #I sort both to avoid ordering issues
                 to_records[key].sort()
                 if from_records[key] != to_records[key]:
-                    log.debug("Updating %s %s %s to %s" % (key[0], key[1], key[2], to_records[key]))
+                    log.info("Updating %s %s %s to %s" % (key[0], key[1], key[2], to_records[key]))
                     updates[key] = [from_records[key], to_records[key]]
                 del from_records[key] #no-op or modify either way pull from from_records.
             else:
                 adds[key] = to_records[key]
-                log.debug("Adding %s %s %s %s" % (key[0], key[1], key[2], to_records[key]))
+                log.info("Adding %s %s %s %s" % (key[0], key[1], key[2], to_records[key]))
 
         #Anything remaining in the from_records is a delete
         for key in from_records.iterkeys():
             deletes[key] = from_records[key]
-            log.debug("Removing %s %s %s %s" % (key[0], key[1], key[2], from_records[key]))
+            log.info("Removing %s %s %s %s" % (key[0], key[1], key[2], from_records[key]))
 
         return adds, deletes, updates
 
@@ -174,6 +196,7 @@ class Zone:
                 continue
 
             rvalue = rdata.to_text()
+            log.debug("Adding %s, type %s, ttl %d, value %s to rrecords" % (name, rtype, ttl, rvalue))
             if (name, rtype, ttl) in rrecords:
                 new_value = rrecords[(name, rtype, ttl)]
                 new_value.append(rvalue)
@@ -201,8 +224,12 @@ class Zone:
             ttl = str(rrecord.ttl)
             simple_bind += "$TTL %s\n" % (ttl)
 
-            for value in rrecord.resource_records:
-                simple_bind += "%s\tIN\t%s\t%s\n" % (name, rtype, str(value))
+            if len(rrecord.resource_records) == 0 and rrecord.alias_hosted_zone_id is not None:
+                simple_bind += "%s\tIN\tTXT\t\"Alias %s %s\"\n" % (name, str(rrecord.alias_hosted_zone_id), \
+                    str(rrecord.alias_dns_name))
+            else:
+                for value in rrecord.resource_records:
+                    simple_bind += "%s\tIN\t%s\t%s\n" % (name, rtype, str(value))
 
             last_type = rtype
             last_name = name
@@ -224,6 +251,7 @@ class Zone:
             else:
                 break
 
+        log.debug("Simple Bind zone created from aws r53 response.\n" + simple_bind )
         zone = dns.zone.from_text(simple_bind, origin=self.zone_name, relativize=False)
         return zone
 
@@ -241,10 +269,10 @@ class Zone:
             nameservers = ""
             for nameserve in zone['DelegationSet']['NameServers']:
                 nameservers += nameserve + ' '
-            log.info('Zone nameservers: ' + nameservers)
+            log.warn('Zone nameservers: ' + nameservers)
             self.id = zone['HostedZone']['Id'].replace('/hostedzone/', '')
             zone_xml = self._create_xml(zone_file)
-            #log.debug(zone_xml)
+            log.debug(zone_xml)
             log.info("Adding rrsets to zone " + self.zone_name)
             for xml in zone_xml:
                 change = self.conn.change_rrsets(self.id, xml)
@@ -255,7 +283,7 @@ class Zone:
         xmlout += self._add_change_xml('.'.join([host, self.zone_name]), rtype, ttl, [value, ])
         xmlout += self.xml_footer
         self.conn.change_rrsets(self.id, xmlout)
-        log.debug("Updating %s %s %s to %s" % (host, rtype, ttl, value))
+        log.info("Updating %s %s %s to %s" % (host, rtype, ttl, value))
 
     def exists(self):
         """Return true if the self.zone_name exists on AWS, false otherwise."""
@@ -304,6 +332,7 @@ class Zone:
         log.info('Updating zone %s' % self.zone_name)
         if not dry_run:
             for changeset in changesets:
+                log.debug(changeset)
                 change = self.conn.change_rrsets(self.id, changeset)
 
     def update_host(self, host, rtype, ttl, existing, value):
@@ -312,7 +341,7 @@ class Zone:
         xmlout += self._update_change_xml('.'.join([host, self.zone_name]) , rtype, ttl, existing[1], [value, ])
         xmlout += self.xml_footer
         self.conn.change_rrsets(self.id, xmlout)
-        log.debug("Updating %s %s %s to %s" % (host, rtype, ttl, value))
+        log.info("Updating %s %s %s to %s" % (host, rtype, ttl, value))
 
     def remove(self, dry_run):
         """Remove this zone from AWS."""
@@ -341,6 +370,7 @@ class Zone:
                     elif rtype == 'SOA':
                         continue
                     change_needed = True
+                    log.debug("Removing %s, type %s, ttl %d, values %s" % (name, rtype, ttl, rrecords[key]))
                     xml += self._delete_change_xml(name, rtype, ttl, rrecords[key])
 
                 xml += self.xml_footer
@@ -348,5 +378,6 @@ class Zone:
                     xmllist.append(xml)
 
             for xml in xmllist:
+                log.debug("Change xml\n" + str(xml))
                 change = self.conn.change_rrsets(self.id, xml)
             delete = self.conn.delete_hosted_zone(self.id)
